
import requests
from django import template
from django.contrib.admin.views.main import PAGE_VAR
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from milea_base import MILEA_VARS

register = template.Library()


@register.simple_tag
def get_setting(name: str):
    """
    Prüft ob die übergebene variable in den core settings existiert
    und gibt den value aus den settings zurück.

    :param name: name of setting variable
    :return: value of setting variable
    """

    split = name.split(".")

    try:
        return MILEA_VARS[split[0]][split[1]]
    except KeyError:
        return None
    except Exception as e:
        raise e

@register.simple_tag
def milea_paginator_number(cl, i):
    """
    Generate an individual page index link in a paginated list.
    """
    if i == cl.paginator.ELLIPSIS:
        return format_html("{} ", cl.paginator.ELLIPSIS)
    else:
        return format_html(
            '<li class="page-item {}"><a class="page-link" href="{}"{}>{}</a></li>',
            'active' if i == cl.page_num else '',
            cl.get_query_string({PAGE_VAR: i}),
            mark_safe(' class="end"' if i == cl.paginator.num_pages else ""),
            i,
        )

@register.simple_tag
def setvar(val=None):
    return val

@register.simple_tag
def get_random_quote():
    """
    Funktion, die das zufällige Zitat von zenquotes.io abruft

    :return: dict with quote of the day
    """
    response = requests.get("https://zenquotes.io/api/random")
    if response.status_code == 200:
        data = response.json()
        return data[0]  # return dict with q (quote) and a (author)
    else:
        return "Willkommen"



from django.apps import apps
from django.template import Library
from django.utils.text import slugify

from milea_base import MILEA_VARS

register = Library()

@register.simple_tag
def app_additional_config(app):
    """
    Get additional infos from the app config and build a second level menu
    """

    menu_icon = "ti ti-package"  # Default Icon
    menu_firstlevel = list()
    menu_secondlevel = list()
    menu_others = False  # Special menu for config apps defined in MILEA_VARS
    app_config = apps.get_app_config(app['app_label'])

    # Menu icon
    if hasattr(app_config, 'menu_icon'):
        menu_icon = getattr(app_config, 'menu_icon')

    # Default Menu
    if not hasattr(app_config, 'menu_firstlvl') and not hasattr(app_config, 'menu_secondlvl'):
        menu_firstlevel = app['models']

    # First Level Menu
    if hasattr(app_config, 'menu_firstlvl'):
        for menu in getattr(app_config, 'menu_firstlvl'):
            for item in app['models']:
                if item['object_name'] in menu:
                    menu_firstlevel.append(item)

    # Second Lvl Menu
    if hasattr(app_config, 'menu_secondlvl'):
        for menu in getattr(app_config, 'menu_secondlvl'):
            menu_tmp = dict(name=menu[0], key=slugify(menu[0]), models=[])
            for item in app['models']:
                if item['object_name'] in menu[1]:
                    menu_tmp['models'].append(item)
            if len(menu_tmp['models']) > 0:
                menu_secondlevel.append(menu_tmp)

    # Config Menu
    if app_config.name in MILEA_VARS['milea_base']['MENUOTHERS']:
        menu_others = True

    r = dict(
        menu_icon=menu_icon,
        menu_firstlevel=menu_firstlevel,
        menu_secondlevel=menu_secondlevel,
        menu_others=menu_others
    )

    return r


@register.simple_tag
def sort_app_list(app_list):
    # Get the order value from settings var
    order = MILEA_VARS['milea_base']['MENUORDER']

    # Create a dictionary mapping app_labels to app_dicts
    app_dict = {app['app_label']: app for app in app_list}

    # Sort the app_list based on the 'order' list
    sorted_app_list = [app_dict[label] for label in order if label in app_dict]

    # Add apps not included in 'order' to the end of the list
    apps_not_in_order = [app for app in app_list if app['app_label'] not in order]
    sorted_app_list.extend(apps_not_in_order)

    return sorted_app_list

@register.simple_tag
def has_others(app_list):
    """Check if user has rights for apps in others menu"""
    for app in app_list:
        if app['app_label'] in MILEA_VARS['milea_base']['MENUOTHERS']:
            return True
    return False



from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django_currentuser.db.models import CurrentUserField


class MileaModel(models.Model):

    OBJ_VERB_TAG = ""

    is_active = models.BooleanField(default=True, db_index=True, verbose_name=_("active"))
    created_at = models.DateTimeField(auto_now=False, auto_now_add=True, verbose_name=_("created at"))
    created_by = CurrentUserField(on_delete=models.SET_NULL, blank=True, editable=False, related_name="%(app_label)s_%(class)s_created_by", verbose_name=_("created by"))
    updated_at = models.DateTimeField(auto_now=True, auto_now_add=False, verbose_name=_("updated at"))
    updated_by = CurrentUserField(on_update=True, on_delete=models.SET_NULL, related_name="%(app_label)s_%(class)s_updated_by", verbose_name=_("updated by"))

    @property
    def verbose_id(self):
        if self.OBJ_VERB_TAG:
            return "%s.%s" % (self.OBJ_VERB_TAG, str(self.id).zfill(6))
        else:
            return "%s" % (str(self.id).zfill(6))
    verbose_id.fget.short_description = _("identifier")

    def get_admin_url(self):
        if self.pk:
            return reverse('admin:%s_%s_change' % (self._meta.app_label, self._meta.model_name), args=[self.pk])
        return ""

    class Meta:
        abstract = True



from django.contrib.admin.models import ADDITION, CHANGE, LogEntry
from django.contrib.admin.utils import construct_change_message
from django.contrib.contenttypes.models import ContentType

from milea_base import MILEA_VARS


def create_log_entry(user_id, obj, new, form=None, msg=None):
    """Creates a Log Entry in the django admin history"""

    change_message = msg if msg is not None else construct_change_message(form, None, True if new else False)
    if user_id is None:
        user_id = 1  # Default System User

    LogEntry.objects.log_action(
        user_id,
        ContentType.objects.get_for_model(obj.__class__).id,
        obj.id,
        str(obj),
        ADDITION if new else CHANGE,
        change_message
    )


def get_setting(name: str):
    """
    Prüft ob die übergebene variable in den core settings existiert
    und gibt den value aus den settings zurück.

    :param name: name of setting variable
    :return: value of setting variable
    """

    split = name.split(".")

    try:
        return MILEA_VARS[split[0]][split[1]]
    except KeyError:
        return None
    except Exception as e:
        raise e



from django.conf import settings as core

MILEA_VARS = {}

for app in core.MILEA_APPS:
    try:
        settings_module = __import__(f"{app}.settings", fromlist=["*"])
    except ImportError:
        raise Exception(f"Error while importing configuration for app {app}: settings.py not found.")
    else:
        vars = {key: getattr(settings_module, key) for key in dir(settings_module) if not key.startswith('__')}
        MILEA_VARS[app] = vars

for app, value in MILEA_VARS.items():
    try:
        vars = core.MILEA_VARS[app]
        for key, val in vars.items():
            if key in MILEA_VARS[app]:
                MILEA_VARS[app][key] = val
    except Exception:
        pass



from django.contrib import admin, messages
from django.core.exceptions import PermissionDenied
from django.db import models
from django.forms import Textarea
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from django_object_actions import DjangoObjectActions, action

from milea_base.utils import create_log_entry, get_setting

# Default Naming
admin.site.site_header = admin.site.site_title = get_setting("milea_base.SHORT_NAME")
admin.site.index_title = 'Dashboard'


# Default Admin
class MileaAdmin(DjangoObjectActions, admin.ModelAdmin):

    show_sysdata = True  # Zeigt die Systemfelder an (created, updated, ...)

    list_display = ('verbose_id',)
    list_display_links = ('verbose_id',)
    milea_readonly_fields = ('created_at', '_created_by', 'updated_at', '_updated_by',)
    admin_fieldsets = ()
    search_fields = ['id',]
    list_per_page = 10

    formfield_overrides = {
        models.TextField: {'widget': Textarea(attrs={'rows': 9})},
    }

    ##################
    # Property Fields
    ##################

    def _created_by(self, obj=None):
        if obj and obj.created_by is not None:
            return str(obj.created_by)
        elif obj and obj.created_by is None:
            return "Systemuser"
        return None
    _created_by.short_description = _("created by")

    def _updated_by(self, obj=None):
        if obj and obj.updated_by is not None:
            return str(obj.updated_by)
        elif obj and obj.updated_by is None:
            return "Systemuser"
        return None
    _updated_by.short_description = _("updated by")

    def is_active_badge(self, obj):
        return format_html(
            '<span class="badge bg-{} me-1"></span> {}', 'success' if obj.is_active else 'secondary', '',
        )
    is_active_badge.short_description = _("active")

    ##################
    # Action Buttons
    ##################

    @action(label=_("inactive"), description=_("Click this Button to activate this object"), attrs={'icon': "ti ti-at-off", 'class': 'btn-inactive btn-danger'})
    def set_to_active(self, request, obj):
        request.user.has_perm("%s.change_%s" % (obj._meta.app_label, obj._meta.model_name))
        if request.user.has_perm("%s.change_%s" % (obj._meta.app_label, obj._meta.model_name)):
            obj.is_active = False if obj.is_active else True
            obj.save()
            create_log_entry(request.user.id, obj, new=False, form=None, msg=_("Status changed to active"))
            messages.success(request, _("%s has been activated") % obj.name)
            return HttpResponseRedirect(reverse('admin:%s_%s_change' % (obj._meta.app_label, obj._meta.model_name), args=[obj.pk]))
        else:
            raise PermissionDenied()

    @action(label=_("active"), description=_("Click this Button to deactivate this object"), attrs={'icon': "ti ti-power", 'class': 'btn-success'})
    def set_to_inactive(self, request, obj):
        if request.user.has_perm("%s.change_%s" % (obj._meta.app_label, obj._meta.model_name)):
            obj.is_active = False if obj.is_active else True
            obj.save()
            create_log_entry(request.user.id, obj, new=False, form=None, msg=_("Status changed to inactive"))
            messages.success(request, _("%s has been deactivated") % obj.name)
            return HttpResponseRedirect(reverse('admin:%s_%s_change' % (obj._meta.app_label, obj._meta.model_name), args=[obj.pk]))
        else:
            raise PermissionDenied()

    def get_change_actions(self, request, object_id, form_url):
        actions = super().get_change_actions(request, object_id, form_url)
        actions = list(actions)

        obj = self.model.objects.get(pk=object_id)
        if obj.is_active and 'set_to_active' in actions:
            actions.remove('set_to_active')
        elif not obj.is_active and 'set_to_inactive' in actions:
            actions.remove('set_to_inactive')

        return actions

    ##################
    # Django / Others
    ##################

    def get_readonly_fields(self, request, obj=None):
        default = super().get_readonly_fields(request, obj)
        return self.milea_readonly_fields + default

    def get_fieldsets(self, request, obj=None):
        fieldsets = super().get_fieldsets(request, obj=obj)

        # Add special fieldsets for superuser
        if request.user.is_superuser:
            fieldsets += self.admin_fieldsets

        # Add system data fieldset
        if self.show_sysdata:
            fieldsets += (
                (_("System data"), {
                    'classes': ('milea-system-data mt-3 col-lg-12',),
                    'fields': (('created_at', '_created_by', 'updated_at', '_updated_by'),),
                }),
            )

        return fieldsets

    def get_fields(self, request, obj=None):
        fields = super().get_fields(request, obj=obj)

        # Remove the fields for the system data row
        fields = [field for field in fields if field not in ('is_active', 'created_at', '_created_by', 'updated_at', '_updated_by')]

        return fields

    def get_search_results(self, request, queryset, search_term):
        # Allow search with verbose_id
        if self.model.OBJ_VERB_TAG and search_term.startswith('%s.' % self.model.OBJ_VERB_TAG):
            search_term = search_term.split('.')
            search_term = int(search_term[1])
        queryset, use_distinct = super().get_search_results(request, queryset, search_term)

        return queryset, use_distinct

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        # Ridiculous override just to get rid of the "Hold down" text. Done is better than perfect...
        form_field = super().formfield_for_manytomany(db_field, request, **kwargs)
        msg = _("Hold down “Control”, or “Command” on a Mac, to select more than one.")
        form_field.help_text = form_field.help_text.replace(str(msg), '').strip()
        return form_field




from setuptools import find_packages, setup

setup(
    name='milea-base',
    version='0.3',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['Django>=5.0'],
    author='red-pepper-services',
    author_email='pypi@schiegg.at',
    description='Milea Framework - Milea Base Module',
    license='MIT',
)
